//------------------------------------------------------
//
//   Greg's Prospecting Mod - Common
//
//------------------------------------------------------

package net.minecraft.src;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Map;
import java.util.Hashtable;
import gcewing.lighting.*;

public abstract class GregsLighting extends BaseMod {

	public static String modName = "GregsLighting";
	public static String channelName = "gce.lighting";
	public static String version = "1.1";
	
	public static Block floodlight;
	public static Block floodlightBeam;

//	public static Item ...;
	
	static String configName = modName + ".cfg";
	static File cfgfile;
	static OrderedProperties config;
	static Map <Integer, String> idToName = new Hashtable <Integer, String>();
	
	public static boolean isServer;
	
	GregsLighting(boolean server) {
		isServer = server;
	}

	public void load(File mcdir) {
		System.out.printf("GregsLighting: load()\n");
		loadConfig(mcdir);
		registerBlocks();
		registerItems();
		addRecipes();
		registerChannels();
		saveConfig();
	}
	
	void loadConfig(File mcdir) {
		File cfgdir = new File(mcdir, "/config/");
		cfgfile = new File(cfgdir, configName);
		config = new OrderedProperties();
		try {
			config.load(new FileInputStream(cfgfile));
		}
		catch (FileNotFoundException e) {
			System.out.printf("%s: No existing config file\n", modName);
		}
		catch (IOException e) {
			System.out.printf("%s: Failed to read %s\n%s\n", modName,
				cfgfile.getPath(), e);
		}
		config.extended = false;
	}
	
	void saveConfig() {
		try {
			if (config.extended) {
				System.out.printf("%s: Writing config file\n", modName);
				config.store(new FileOutputStream(cfgfile),
					modName + " Configuration File");
			}
		}
		catch (IOException e) {
			System.out.printf("%s: Failed to %s\n%s\n", modName,
				cfgfile.getPath(), e);
		}
	}

	public String getVersion() {
		return version;
	}
	
	void registerChannels() {
//		ModLoader.registerPacketChannel(this, channelName);
	}
	
	void registerBlocks() {
		System.out.printf("%s: registerBlocks()\n", modName);
		floodlightBeam = addBlock("Floodlight Beam", new BlockFloodlightBeam(getBlockID(198, "floodlightBeam")));
		floodlight = addBlock("Floodlight", new BlockFloodlight(getBlockID(199, "floodlight")));
	}
	
	void registerItems() {
//		battery = addItem("Battery", newItemSI(3001, "battery", 6, "battery.png"));
	}

	void addRecipes() {
//		addShapelessRecipe(battery, 1, batteryCasing, electrolytePaste, carbonRod);
//		addSmeltingRecipe(carbonFilament, 1, Item.silk);
//		addRecipe(cathodeRayTube, 1, "|gG", "|gf", "|gG",
//			'|', Block.thinGlass, 'g', Item.lightStoneDust, 'G', Block.glass, 'f', carbonFilament);
			addRecipe(floodlight, 1, "IrI", "IgI", "GGG",
				'I', Item.ingotIron, 'r', Item.redstone, 'g', Item.lightStoneDust, 'G', Block.glass);
	}
	
	void addRecipe(Item product, int qty, Object... params) {
		ModLoader.addRecipe(new ItemStack(product, qty), params);
	}

	void addRecipe(Block product, int qty, Object... params) {
		ModLoader.addRecipe(new ItemStack(product, qty), params);
	}

	void addShapelessRecipe(Item product, int qty, Object... params) {
		ModLoader.addShapelessRecipe(new ItemStack(product, qty), params);
	}

	void addSmeltingRecipe(Item product, int qty, Item input) {
		ModLoader.addSmelting(input.shiftedIndex, new ItemStack(product, qty));
	}
	
	void addSmeltingRecipe(Item product, int qty, Block input) {
		ModLoader.addSmelting(input.blockID, new ItemStack(product, qty));
	}
	
	static int getBlockID(int defaultID, String name) {
		return getBlockOrItemID(defaultID, "block." + name, 0);
	}
	
	static int getItemID(int defaultID, String name) {
		return getBlockOrItemID(defaultID, "item." + name, 256);
	}
	
	static int getBlockOrItemID(int defaultID, String name, int offset) {
		String key = name + ".id";
		int id;
		if (config.containsKey(key)) {
			String value = (String) config.get(key);
			id = Integer.parseInt(value);
		}
		else {
			config.put(key, Integer.toString(defaultID));
			id = defaultID;
		}
		idToName.put(Integer.valueOf(offset + id), name);
		return id;
	}

	Item newItem(int id, String name) {
		return new Item(getItemID(id, name));
	}
	
	Item newItemI(int id, String name, String icon) {
		return new Item(getItemID(id, name)).setIconIndex(iconOverride(icon));
	}
	
	Item newItemSI(int id, String name, int maxStack, String icon) {
		return new Item(getItemID(id, name)).setMaxStackSize(maxStack).setIconIndex(iconOverride(icon));
	}
	
	int iconOverride(String name) {
		return ModLoader.addOverride("/gui/items.png", "/gcewing/prospecting/resources/" + name);
	}
	
	Block addBlock(String name, Block block) {
		System.out.printf("%s: Adding block %s id %s\n", modName, name, block.blockID);
		block.setBlockName("gcewing." + idToName.get(block.blockID));
		ModLoader.registerBlock(block);
		ModLoader.addName(block, name);
		return block;
	}

	Item addItem(String name, Item item) {
		item.setItemName("gcewing." + idToName.get(item.shiftedIndex));
		ModLoader.addName(item, name);
		return item;
	}
	
	//-----------------------------------------------------------------------------------------
	
	public static void updateFloodlightBlock(World world, int x, int y, int z) {
		// Neighbourhood of a floodlight block at x,y,z has changed.
		boolean powered = floodlightActiveAt(world, x, y, z);
		int oldData = world.getBlockMetadata(x, y, z);
		int newData = powered ? 1 : 0;
		if (oldData != newData) {
			world.setBlockMetadata(x, y, z, newData);
			world.markBlockNeedsUpdate(x, y, z);
		}
		if (powered)
			projectFloodlightBeam(world, x, y-1, z);
		else
			removeFloodlightBeam(world, x, y-1, z);
	}
	
	public static void updateFloodlightBeamBlock(World world, int x, int y, int z) {
		// Neighbourhood of a floodlight beam block at x,y,z has changed.
		if (floodlightActiveAt(world, x, y+1, z))
			projectFloodlightBeam(world, x, y-1, z);
		else
			removeFloodlightBeam(world, x, y, z);
	}
	
	static boolean floodlightActiveAt(World world, int x, int y, int z) {
		// True if the block at x,y,z is either a powered floodlight block or a
		// floodlight beam block, or is illuminated by such a block from above
		// through transparent blocks.
		int h = world.getHeight();
		while (y < h) {
			int id = world.getBlockId(x, y, z);
			if (id == floodlight.blockID)
				return world.isBlockIndirectlyGettingPowered(x, y, z);
			else if (id == floodlightBeam.blockID)
				return true;
			else if (id == 0 || Block.blocksList[id].isOpaqueCube())
				return false;
			y += 1;
		}
		return false;
	}

	static void projectFloodlightBeam(World world, int x, int y, int z) {
		// Add floodlight beam blocks downwards starting from the given coords
		// until an opaque block is found.
		traceFloodlightBeam(world, x, y, z, 0, floodlightBeam.blockID);
	}
	
	static void removeFloodlightBeam(World world, int x, int y, int z) {
		// Remove floodlight beam blocks downwards starting from the given coords
		// until an empty or opaque block is found.
		traceFloodlightBeam(world, x, y, z, floodlightBeam.blockID, 0);
	}
	
	static void traceFloodlightBeam(World world, int x, int y, int z, int oldID, int newID) {
		while (y >= 0) {
			int id = world.getBlockId(x, y, z);
			if (id == oldID)
				world.setBlockWithNotify(x, y, z, newID);
			else if (id != 0 && Block.blocksList[id].isOpaqueCube())
				break;
			y -= 1;
		}
	}
	
}
